@@ -21,8 +21,6 @@
 """
 Command-line interface to the OpenStack Zun API.
 """
-
-from __future__ import print_function
 import argparse
 import getpass
 import logging
@@ -32,7 +30,16 @@ import sys
 from oslo_utils import encodeutils
 from oslo_utils import importutils
 from oslo_utils import strutils
-import six
+
+from zunclient import api_versions
+from zunclient import client as base_client
+from zunclient.common.apiclient import auth
+from zunclient.common import cliutils
+from zunclient import exceptions as exc
+from zunclient.i18n import _
+from zunclient.v1 import shell as shell_v1
+from zunclient import version
+
 
 profiler = importutils.try_import("osprofiler.profiler")
 
@@ -52,15 +59,6 @@ try:
 except ImportError:
     pass
 
-from zunclient import api_versions
-from zunclient import client as base_client
-from zunclient.common.apiclient import auth
-from zunclient.common import cliutils
-from zunclient import exceptions as exc
-from zunclient.i18n import _
-from zunclient.v1 import shell as shell_v1
-from zunclient import version
-
 DEFAULT_API_VERSION = api_versions.DEFAULT_API_VERSION
 DEFAULT_ENDPOINT_TYPE = 'publicURL'
 DEFAULT_SERVICE_TYPE = 'container'
@@ -335,7 +333,8 @@ class OpenStackZunShell(object):
                                 'ZUN_API_VERSION',
                                 default=DEFAULT_API_VERSION),
                             help='Accepts X, X.Y (where X is major, Y is minor'
-                                 ' part), defaults to env[ZUN_API_VERSION].')
+                                 ' part) or "X.latest", defaults to'
+                                 ' env[ZUN_API_VERSION].')
         parser.add_argument('--zun_api_version',
                             help=argparse.SUPPRESS)
 
@@ -345,6 +344,18 @@ class OpenStackZunShell(object):
                             help='Specify a CA bundle file to use in '
                             'verifying a TLS (https) server certificate. '
                             'Defaults to env[OS_CACERT].')
+        parser.add_argument('--os-cert',
+                            metavar='<ca-certificate>',
+                            default=cliutils.env('OS_CERT', default=None),
+                            help='Specify a client certificate file (for '
+                                 'client auth). '
+                                 'Defaults to env[OS_CERT].')
+        parser.add_argument('--os-key',
+                            metavar='<ca-certificate>',
+                            default=cliutils.env('OS_KEY', default=None),
+                            help='Specify a client certificate key file (for '
+                                 'client auth). '
+                                 'Defaults to env[OS_KEY].')
 
         parser.add_argument('--bypass-url',
                             metavar='<bypass-url>',
@@ -517,8 +528,9 @@ class OpenStackZunShell(object):
             spot = argv.index('--endpoint_type')
             argv[spot] = '--endpoint-type'
 
+        do_help = "help" in args
         subcommand_parser = self.get_subcommand_parser(
-            api_version, do_help=("help" in args))
+            api_version, do_help=do_help)
 
         self.parser = subcommand_parser
 
@@ -541,12 +553,13 @@ class OpenStackZunShell(object):
          os_user_domain_id, os_user_domain_name,
          os_project_domain_id, os_project_domain_name,
          os_auth_url, os_auth_system, endpoint_type,
-         service_type, bypass_url, insecure) = (
+         service_type, bypass_url, insecure, os_cacert, os_cert, os_key) = (
             (args.os_username, args.os_project_name, args.os_project_id,
              args.os_user_domain_id, args.os_user_domain_name,
              args.os_project_domain_id, args.os_project_domain_name,
              args.os_auth_url, args.os_auth_system, args.endpoint_type,
-             args.service_type, args.bypass_url, args.insecure)
+             args.service_type, args.bypass_url, args.insecure,
+             args.os_cacert, args.os_cert, args.os_key)
         )
 
         if os_auth_system and os_auth_system != "keystone":
@@ -624,6 +637,42 @@ class OpenStackZunShell(object):
 
         client = base_client
 
+        if not do_help:
+            if api_version.is_latest():
+                # This client is just used to discover api version.
+                # Version API needn't microversion, so we just pass
+                # version 1.1 at here.
+                self.cs = client.Client(
+                    version=api_versions.APIVersion("1.1"),
+                    username=os_username,
+                    password=os_password,
+                    project_id=os_project_id,
+                    project_name=os_project_name,
+                    user_domain_id=os_user_domain_id,
+                    user_domain_name=os_user_domain_name,
+                    project_domain_id=os_project_domain_id,
+                    project_domain_name=os_project_domain_name,
+                    auth_url=os_auth_url,
+                    service_type=service_type,
+                    region_name=args.os_region_name,
+                    endpoint_override=bypass_url,
+                    interface=endpoint_type,
+                    insecure=insecure,
+                    cacert=os_cacert)
+                api_version = api_versions.discover_version(self.cs,
+                                                            api_version)
+
+            min_version = api_versions.APIVersion(api_versions.MIN_API_VERSION)
+            max_version = api_versions.APIVersion(api_versions.MAX_API_VERSION)
+            if not api_version.matches(min_version, max_version):
+                raise exc.CommandError(
+                    _("The specified version isn't supported by "
+                      "client. The valid version range is '%(min)s' "
+                      "to '%(max)s'") % {
+                        "min": min_version.get_string(),
+                        "max": max_version.get_string()}
+                )
+
         kwargs = {}
         if profiler:
             kwargs["profile"] = args.profile
@@ -643,6 +692,9 @@ class OpenStackZunShell(object):
                                 endpoint_override=bypass_url,
                                 interface=endpoint_type,
                                 insecure=insecure,
+                                cacert=os_cacert,
+                                cert=os_cert,
+                                key=os_key,
                                 **kwargs)
 
         args.func(self.cs, args)
@@ -711,7 +763,7 @@ def main():
             map(encodeutils.safe_decode, sys.argv[1:]))
     except Exception as e:
         logger.debug(e, exc_info=1)
-        print("ERROR: %s" % encodeutils.safe_encode(six.text_type(e)),
+        print("ERROR: %s" % encodeutils.safe_encode(str(e)),
               file=sys.stderr)
         sys.exit(1)
 
