@@ -11,9 +11,11 @@
 #    under the License.
 
 import copy
-from six.moves.urllib import parse
+
 import testtools
 from testtools import matchers
+from urllib import parse
+from zunclient.common import utils as zun_utils
 from zunclient import exceptions
 from zunclient.tests.unit import utils
 from zunclient.v1 import containers
@@ -36,7 +38,9 @@ CONTAINER1 = {'id': '1234',
               'runtime': 'runc',
               'hostname': 'testhost',
               'disk': '20',
-              'auto_heal': False
+              'auto_heal': False,
+              'privileged': False,
+              'healthcheck': {}
               }
 
 CONTAINER2 = {'id': '1235',
@@ -56,7 +60,9 @@ CONTAINER2 = {'id': '1235',
               'auto_remove': False,
               'runtime': 'runc',
               'hostname': 'testhost',
-              'auto_heal': False
+              'auto_heal': False,
+              'privileged': True,
+              'healthcheck': {}
               }
 
 NETWORK1 = {'net_id': '99e90853-e1fd-4c57-a116-9e335deaa592',
@@ -285,7 +291,7 @@ fake_responses = {
     {
         'GET': (
             {},
-            None,
+            {'data': data},
         ),
     },
     '/v1/containers/%s/put_archive?%s'
@@ -293,7 +299,7 @@ fake_responses = {
     {
         'POST': (
             {},
-            {'data': data},
+            None,
         ),
     },
     '/v1/containers/%s/stats?%s'
@@ -649,25 +655,25 @@ class ContainerManagerTest(testtools.TestCase):
         self.assertIsNone(containers)
 
     def test_containers_get_archive(self):
-        containers = self.mgr.get_archive(CONTAINER1['id'], path)
+        response = self.mgr.get_archive(CONTAINER1['id'], path)
         expect = [
             ('GET', '/v1/containers/%s/get_archive?%s'
              % (CONTAINER1['id'], parse.urlencode({'path': path})),
              {'Content-Length': '0'}, None)
         ]
         self.assertEqual(expect, self.api.calls)
-        self.assertIsNone(containers)
+        self.assertEqual(zun_utils.decode_file_data(data), response['data'])
 
     def test_containers_put_archive(self):
-        containers = self.mgr.put_archive(CONTAINER1['id'], path, data)
+        response = self.mgr.put_archive(CONTAINER1['id'], path, data)
         expect = [
             ('POST', '/v1/containers/%s/put_archive?%s'
              % (CONTAINER1['id'], parse.urlencode({'path': path})),
              {'Content-Length': '0'},
-             {'data': data})
+             {'data': zun_utils.encode_file_data(data)})
         ]
         self.assertEqual(expect, self.api.calls)
-        self.assertTrue(containers)
+        self.assertTrue(response)
 
     def test_containers_commit(self):
         containers = self.mgr.commit(CONTAINER1['id'], repo, tag)
