@@ -15,12 +15,13 @@
 import argparse
 from contextlib import closing
 import io
-import json
 import os
 import tarfile
 import time
 import yaml
 
+from oslo_serialization import jsonutils
+
 from zunclient.common import cliutils as utils
 from zunclient.common import utils as zun_utils
 from zunclient.common.websocketclient import exceptions
@@ -28,12 +29,18 @@ from zunclient.common.websocketclient import websocketclient
 from zunclient import exceptions as exc
 
 
-DEPRECATION_MESSAGE = (
+RENAME_DEPRECATION_MESSAGE = (
     'WARNING: Rename container command deprecated and will be removed '
     'in a future release.\nUse Update container command to avoid '
     'seeing this message.')
 
 
+SG_DEPRECATION_MESSAGE = (
+    'WARNING: Security group related commands deprecated and will be removed '
+    'in a future release.\nUse Neutron commands to manage security groups '
+    'instead.')
+
+
 def _show_container(container):
     zun_utils.format_container_addresses(container)
     utils.print_dict(container._info)
@@ -50,6 +57,20 @@ def _show_container(container):
     required=False, metavar='<restart>',
     help='Restart policy to apply when a container exits'
          '(no, on-failure[:max-retry], always, unless-stopped)')
+@utils.exclusive_arg(
+    'secgroup_expose_port',
+    '--security-group',
+    metavar='<security-group>',
+    action='append', default=[],
+    help='The name of security group for the container. '
+         'May be used multiple times.')
+@utils.exclusive_arg(
+    'secgroup_expose_port',
+    '-p', '--expose-port',
+    action='append',
+    default=[],
+    metavar='<port>',
+    help='Expose container port(s) to outside (format: <port>[/<protocol>])')
 @utils.arg('-n', '--name',
            metavar='<name>',
            help='name of the container')
@@ -95,11 +116,6 @@ def _show_container(container):
                 'It can have following values: '
                 '"docker": pull the image from Docker Hub. '
                 '"glance": pull the image from Glance. ')
-@utils.arg('--security-group',
-           metavar='<security-group>',
-           action='append', default=[],
-           help='The name of security group for the container. '
-                'May be used multiple times.')
 @utils.arg('command',
            metavar='<command>',
            nargs=argparse.REMAINDER,
@@ -148,6 +164,34 @@ def _show_container(container):
            action='store_true',
            default=False,
            help='The flag of healing non-existent container in docker.')
+@utils.arg('--privileged',
+           dest='privileged',
+           action='store_true',
+           default=False,
+           help='Give extended privileges to this container')
+@utils.arg('--healthcheck',
+           action='append',
+           default=[],
+           metavar='<cmd=command,interval=time,retries=integer,timeout=time>',
+           help='Specify a test cmd to perform to check that the container'
+                'is healthy. '
+                'cmd: Command to run to check health. '
+                'interval: Time between running the check (s|m|h)'
+                '          (default 0s). '
+                'retries: Consecutive failures needed to report unhealthy. '
+                'timeout: Maximum time to allow one check to run (s|m|h)'
+                '         (default 0s).')
+@utils.arg('--registry',
+           metavar='<registry>',
+           help='The container image registry ID or name')
+@utils.arg('--host',
+           metavar='<host>',
+           help='Requested host to create containers. Admin only by default.'
+                '(Supported by API versions 1.39 or above)')
+@utils.arg('--entrypoint',
+           metavar='<entrypoint>',
+           help='The entrypoint which overwrites the default ENTRYPOINT '
+                'of the image. (Supported by API versions 1.40 or above)')
 def do_create(cs, args):
     """Create a container."""
     opts = {}
@@ -168,15 +212,26 @@ def do_create(cs, args):
     opts['hostname'] = args.hostname
     opts['disk'] = args.disk
     opts['availability_zone'] = args.availability_zone
-    opts['auto_heal'] = args.auto_heal
     opts['command'] = args.command
+    opts['registry'] = args.registry
+    opts['host'] = args.host
+    if args.entrypoint:
+        opts['entrypoint'] = zun_utils.parse_entrypoint(args.entrypoint)
+    if args.healthcheck:
+        opts['healthcheck'] = zun_utils.parse_health(args.healthcheck)
 
+    if args.auto_heal:
+        opts['auto_heal'] = args.auto_heal
     if args.security_group:
         opts['security_groups'] = args.security_group
+    if args.expose_port:
+        opts['exposed_ports'] = zun_utils.parse_exposed_ports(args.expose_port)
     if args.restart:
         opts['restart_policy'] = zun_utils.check_restart_policy(args.restart)
     if args.interactive:
         opts['interactive'] = True
+    if args.privileged:
+        opts['privileged'] = True
     opts = zun_utils.remove_null_parms(**opts)
     _show_container(cs.containers.create(**opts))
 
@@ -306,7 +361,7 @@ def do_show(cs, args):
     opts = zun_utils.remove_null_parms(**opts)
     container = cs.containers.get(**opts)
     if args.format == 'json':
-        print(json.dumps(container._info, indent=4, sort_keys=True))
+        print(jsonutils.dumps(container._info, indent=4, sort_keys=True))
     elif args.format == 'yaml':
         print(yaml.safe_dump(container._info, default_flow_style=False))
     elif args.format == 'table':
@@ -538,6 +593,20 @@ def do_kill(cs, args):
     required=False, metavar='<restart>',
     help='Restart policy to apply when a container exits'
          '(no, on-failure[:max-retry], always, unless-stopped)')
+@utils.exclusive_arg(
+    'secgroup_expose_port',
+    '--security-group',
+    metavar='<security-group>',
+    action='append', default=[],
+    help='The name of security group for the container. '
+         'May be used multiple times.')
+@utils.exclusive_arg(
+    'secgroup_expose_port',
+    '-p', '--expose-port',
+    action='append',
+    default=[],
+    metavar='<port>',
+    help='Expose container port(s) to outside (format: <port>[/<protocol>])')
 @utils.arg('-n', '--name',
            metavar='<name>',
            help='name of the container')
@@ -582,11 +651,6 @@ def do_kill(cs, args):
                 'It can have following values: '
                 '"docker": pull the image from Docker Hub. '
                 '"glance": pull the image from Glance. ')
-@utils.arg('--security-group',
-           metavar='<security-group>',
-           action='append', default=[],
-           help='The name of security group for the container. '
-                'May be used multiple times.')
 @utils.arg('command',
            metavar='<command>',
            nargs=argparse.REMAINDER,
@@ -635,6 +699,34 @@ def do_kill(cs, args):
            action='store_true',
            default=False,
            help='The flag of healing non-existent container in docker.')
+@utils.arg('--privileged',
+           dest='privileged',
+           action='store_true',
+           default=False,
+           help='Give extended privileges to this container')
+@utils.arg('--healthcheck',
+           action='append',
+           default=[],
+           metavar='<cmd=command,interval=time,retries=integer,timeout=time>',
+           help='Specify a test cmd to perform to check that the container'
+                'is healthy. '
+                'cmd: Command to run to check health. '
+                'interval: Time between running the check (s|m|h)'
+                '          (default 0s). '
+                'retries: Consecutive failures needed to report unhealthy. '
+                'timeout: Maximum time to allow one check to run (s|m|h)'
+                '         (default 0s).')
+@utils.arg('--registry',
+           metavar='<registry>',
+           help='The container image registry ID or name')
+@utils.arg('--host',
+           metavar='<host>',
+           help='Requested host to run containers. Admin only by default.'
+                '(Supported by API versions 1.39 or above)')
+@utils.arg('--entrypoint',
+           metavar='<entrypoint>',
+           help='The entrypoint which overwrites the default ENTRYPOINT '
+                'of the image. (Supported by API versions 1.40 or above)')
 def do_run(cs, args):
     """Run a command in a new container."""
     opts = {}
@@ -655,15 +747,26 @@ def do_run(cs, args):
     opts['hostname'] = args.hostname
     opts['disk'] = args.disk
     opts['availability_zone'] = args.availability_zone
-    opts['auto_heal'] = args.auto_heal
     opts['command'] = args.command
+    opts['registry'] = args.registry
+    opts['host'] = args.host
+    if args.entrypoint:
+        opts['entrypoint'] = zun_utils.parse_entrypoint(args.entrypoint)
+    if args.healthcheck:
+        opts['healthcheck'] = zun_utils.parse_health(args.healthcheck)
 
+    if args.auto_heal:
+        opts['auto_heal'] = args.auto_heal
     if args.security_group:
         opts['security_groups'] = args.security_group
+    if args.expose_port:
+        opts['exposed_ports'] = zun_utils.parse_exposed_ports(args.expose_port)
     if args.restart:
         opts['restart_policy'] = zun_utils.check_restart_policy(args.restart)
     if args.interactive:
         opts['interactive'] = True
+    if args.privileged:
+        opts['privileged'] = True
     opts = zun_utils.remove_null_parms(**opts)
     container = cs.containers.run(**opts)
     _show_container(container)
@@ -698,12 +801,26 @@ def do_run(cs, args):
 @utils.arg('--name',
            metavar='<name>',
            help='The new name for the container')
+@utils.exclusive_arg(
+    'auto_heal_value',
+    '--auto-heal',
+    required=False, action='store_true',
+    help='Automatic recovery the status of contaier')
+@utils.exclusive_arg(
+    'auto_heal_value',
+    '--no-auto-heal',
+    required=False, action='store_true',
+    help='Needless recovery the status of contaier')
 def do_update(cs, args):
     """Update one or more attributes of the container."""
     opts = {}
     opts['memory'] = args.memory
     opts['cpu'] = args.cpu
     opts['name'] = args.name
+    if 'auto_heal' in args and args.auto_heal:
+        opts['auto_heal'] = True
+    if 'no_auto_heal' in args and args.no_auto_heal:
+        opts['auto_heal'] = False
     opts = zun_utils.remove_null_parms(**opts)
     if not opts:
         raise exc.CommandError("You must update at least one property")
@@ -711,7 +828,7 @@ def do_update(cs, args):
     _show_container(container)
 
 
-@utils.deprecated(DEPRECATION_MESSAGE)
+@utils.deprecated(RENAME_DEPRECATION_MESSAGE)
 @utils.arg('container',
            metavar='<container>',
            help='ID or name of the container to rename.')
@@ -735,19 +852,28 @@ def do_attach(cs, args):
 @utils.arg('container',
            metavar='<container>',
            help='ID or name of the container to display processes.')
-@utils.arg('ps_args',
-           metavar='<ps_args>',
-           nargs=argparse.REMAINDER,
-           help='The args of the ps command.')
+@utils.arg('--pid',
+           metavar='<pid>',
+           action='append', default=[],
+           help='The args of the ps id.')
 def do_top(cs, args):
     """Display the running processes inside the container."""
-    output = cs.containers.top(args.container, ' '.join(args.ps_args))
+
+    if args.pid:
+        # List container single ps id top result
+        output = cs.containers.top(args.container, ' '.join(args.pid))
+    else:
+        # List container all processes top result
+        output = cs.containers.top(args.container)
     for titles in output['Titles']:
         print("%-20s") % titles,
-    for process in output['Processes']:
+    if output['Processes']:
+        for process in output['Processes']:
+            print("")
+            for info in process:
+                print("%-20s") % info,
+    else:
         print("")
-        for info in process:
-            print("%-20s") % info,
 
 
 @utils.arg('source',
@@ -770,7 +896,7 @@ def do_cp(cs, args):
 
         res = cs.containers.get_archive(**opts)
         dest_path = args.destination
-        tardata = io.BytesIO(res['data'].encode())
+        tardata = io.BytesIO(res['data'])
         with closing(tarfile.open(fileobj=tardata)) as tar:
             tar.extractall(dest_path)
 
@@ -823,6 +949,7 @@ def do_commit(cs, args):
               {'container': args.container, 'e': e})
 
 
+@utils.deprecated(SG_DEPRECATION_MESSAGE)
 @utils.arg('container',
            metavar='<container>',
            help='ID or name of the container to add security group.')
@@ -914,6 +1041,7 @@ def do_network_list(cs, args):
     zun_utils.list_container_networks(networks)
 
 
+@utils.deprecated(SG_DEPRECATION_MESSAGE)
 @utils.arg('container',
            metavar='<container>',
            help='ID or name of the container to remove security group.')
